#define VFOduino     // si activé cela définit les paramètres d'entrées sortie de la carte VFOduino de F5RCT, dans le cas contraire c'est le Raduino de VU2ESE
#define VERSION  "220216" // suppression des spurious au relaché mode TUNE, delay de 100 après coupure de la tonalité.
#define VersID 3 //met à jour les valeurs par défaut de l'EEPROM si ce numéro diffère de celui qui est en EEPROM
// Le croquis utilise 22888 octets (74%) de l'espace de stockage de programmes. 
// Les variables globales utilisent 998 octets (48%) de mémoire dynamique



/**
 * This source file is under General Public License version 5.8 changed by F5RCT, orginal from VU2ESE Ashhar Farhan
 * 
 * IMPORTANT  :  A7 is to implement both straight and paddle keyers. 
 *                Analog input MUST bepulled high by an external resistor of 4.7K from 5 volts.
 *                
 *               This code includes specific Si5351A compact sub routiles from Jerry Gaffke's, KE7ER
 *               loading any Si5351 library is no more necessary !
 * 
 *               To changing the mode SSB/CW is not automatic, user has to change in menu mode !
 *               
 *  FUNCTION button : - Press an hold at power up : loads defaults values and enters in calibarion mode (FCT to escape or PTT to valid)
 *                    - Short press : menu mode, choose with potentiometer,...
 *                    - Hold press and trun potentiometer for croase tuining
 *                    - Hold press > 3 sec VFOA=VFOB and store in  EEPROM
 *  
 * Most source code are meant to be understood by the compilers and the computers. 
 * Code that has to be hackable needs to be well understood and properly documented. 
 * 
 * The RADUINO is a small board that includes the Arduin Nano, a 16x2 LCD display and
 * an Si5351a frequency synthesizer. This board is manufactured by Paradigm Ecomm Pvt Ltd
 * To learn more about Raduino you may visit http://www.hfsigs.com/ 
 * Join dicussion group : https://groups.io/g/bitx20
 * 
 * The Arduino ( www.arduino.cc )works by firt executing the code in a function called setup() and then it 
 * repeatedly keeps calling loop() forever. All the initialization code is kept in setup()
 * and code to continuously sense the tuning knob, the function button, transmit/receive,
 * etc is all in the loop() function. If you wish to study the code top down, then scroll
 * to the bottom of this file and read your way up.
 * 
 * Below are the libraries to be included for building the Raduino 
 * The EEPROM.h library is used to store settings like the frequency memory, caliberation data, 
 * The Wire.h library is used to coounicate in I2C with teh Si5351
 * We include the library and declare the configuration of the LCD of a standard 16x2 LCD panel
 */
#include <EEPROM.h>
#include <Wire.h>
#include <LiquidCrystal.h>
LiquidCrystal lcd(8,9,10,11,12,13);


/*
 * There are two sets of completely programmable pins on the Raduino.
 * First, on the top of the board, in line with the LCD connector is an 8-pin connector
 * that is largely meant for analog inputs and front-panel control. It has a regulated 5v output,
 * ground and six pins. Each of these six pins can be individually programmed 
 * either as an analog input, a digital input or a digital output.
 * The A4 and A5 pins are missing from this connector as they are used to talk to the Si5351 over I2C protocol. 
 * The pins are assigned as follows: 
 *   A0, A1, A2, A3, +5v, GND, A6, A7 (while holding the board up so that back of the board faces you)  
 *      
 * Though, this can be assigned anyway, for this application of the Arduino, we will make the following assignment :
 * 
 *  A7 is to implement both straight and paddle keyers. Analog input pulled high by an external resistor of 4.7K from 5 volts.
 *  
 *  A6 is connected to a center pin of good quality 100K or 10K linear potentiometer with the two other ends connected to
 * ground and +5v lines available on the same connector. This implments the tuning and fucntion menu choice.
 * 
 *  A3  Digital input is connected to a momentarily push button to ground. Fucntions : RIT/Bandswitching, etc.
 *  
 *  A2 is used as voltmeter input to monitor the battery from 0 to 25.6 V connect a divider {4k7//0.1µF} to (GND, A2); {15k+10k adj.} to (A2, input)
 *  
 */

#define PTT  (A0)   // Input to the PTT line of the mic connector, internally pulled up 
#define ANALOG_SMETER  (A1)  // Smeter analog input 0 to 5 V max, don't overdrive this pin !
#define FBUTTON  (A2)  // Digital Input Pullup is connected to a fucntion button: RIT/Bandswitching, etc.
#define ANALOG_TUNING (A3)   // Analog input tunning ontrol 
#define ANALOG_KEYER  (A6) // Analog input MUST BE pulled high by an EXTERNAL resistor of 4.7K from 5 volts.
#define ANALOG_VMETER (A7) // Analog input for Voltmeter : connect a divider {4k7//0.1µF} to (GND, A2); {15k+10k adj.} to (A2, input)
#define VoltmeterCal     147 // calibration factor for Voltmeter
#define SmeterCalDefault  220 // calibration factor for Smeter

/** 
 *  The second set of 16 pins on the bottom connector are have the three clock outputs and the digital lines to control the rig.
 *  This assignment is as follows :
 *    Pin   1   2    3    4    5    6    7    8    9    10   11   12   13   14   15   16
 *         GND +5V CLK0  GND  GND  CLK1 GND  GND  CLK2  GND  D2   D3   D4   D5   D6   D7  
 *
 * LCD display in 4 bit mode. The 4 bit mode requires 4 data lines and two control lines to work:
 * Lines used are : RESET, ENABLE, D4, D5, D6, D7 
 *  These too are flexible with what you may do with them, for the Raduino, we use them to :
 *  - TX_RX line : Switches between Transmit and Receive after sensing the PTT or the morse keyer
 *  - CW_TONE : sinde tone output when key is down
 *  - LPF20M_40M : control LPF relay active low for 20m, rest at high for 40m
 *  - LPF10M_2040M : control LPF relay active low for 10m, rest at high for 20m and 40m
 *  - CW_FILTER : low for SSB filter High for CW filter
 *  - CW_CARRIER line : high turns on the carrier for CW
 */
#define CW_CARRIER (2)    // output high turns on the carrier for CW, remain low for SSB and Rx
#define CW_FILTER (3)     // CW filter active high output, low for SSB filter

#define P3D4  4
#define P3D5  5

#define CW_TONE (6)       // tone burst output when key down
#define TX_RX (7)         // Tx ouput active high, receive low


/**
 * The Arduino, unlike C/C++ on a regular computer with gigabytes of RAM, has very little memory.
 * We have to be very careful with variables that are declared inside the functions as they are 
 * created in a memory region called the stack. The stack has just a few bytes of space on the Arduino
 * if you declare large strings inside functions, they can easily exceed the capacity of the stack
 * and mess up your programs. 
 * We circumvent this by declaring a few global buffers as  kitchen counters where we can 
 * slice and dice our strings. These strings are mostly used to control the display or handle
 * the input and output from the USB port. We must keep a count of the bytes used while reading
 * the serial port as we can easily run out of buffer space. This is done in the serial_in_count variable.
 */
char c[17], b[17];


/**
    The  the Si5351a chip  generates upto three oscillators of various frequencies in the
    Raduino. To learn more about Si5351a you can download the datasheet from www.silabs.com
   ...although, strictly speaking it is not a requirement to understand this code.

    We no longer use the standard SI5351 library because of its huge overhead due to many unused
    features consuming a lot of program space (10 ko). Instead of depending on an external library we now use
    Jerry Gaffke's, KE7ER, lightweight standalone mimimalist "si5351bx" routines (see further down the
    code). Here are some defines and declarations used by Jerry's routines:
*/
#define BB0(x) ((uint8_t)x)             // Bust int32 into Bytes
#define BB1(x) ((uint8_t)(x>>8))
#define BB2(x) ((uint8_t)(x>>16))

#define SI5351BX_ADDR 0x60              // I2C address of Si5351   (typical)
#define SI5351BX_XTALPF 3               // 1:6pf  2:8pf  3:10pf

// If using 27mhz crystal, set XTAL=27000000, MSA=33.  Then vco=891mhz
#define SI5351BX_XTAL 25000000          // Crystal freq in Hz
#define SI5351BX_MSA  35                // VCOA is at 25MHz*35 = 875MHz

// User program may have reason to poke new values into these 3 RAM variables
uint32_t si5351bx_vcoa = (SI5351BX_XTAL*SI5351BX_MSA);  // 25mhzXtal calibrate default
uint8_t  si5351bx_rdiv = 0;             // 0-7, CLK pin sees fout/(2**rdiv)
uint8_t  si5351bx_drive[3] = {1, 1, 1}; // 0=2ma 1=4ma 2=6ma 3=8ma for CLK 0,1,2
uint8_t  si5351bx_clken = 0xFF;         // Private, all CLK output drivers off

// modify this section to affect outputs of Si5351
#define OutBFO  2 //clk 2 X3 output
#define OutLO2  1 //clk 1 X2 output
#define OutLO1  0 //clk 0 X1 output

// Low pass filter and I2C to external PA
#define Thresh160m 2100000L  // 2.0 MHz
#define Thresh80m  4000000L  // 3.8 MHz
#define Thresh40m  7500000L  // 5,35 - 7,2 MHz
#define Thresh30m 10200000L  // 10,2 MHz
#define Thresh20m 14500000L  // 14,5 MHz
#define Thresh15m 21500000L  // 21,5 MHz
#define PaAddress 0x68
unsigned long Hysth160m = Thresh160m;
unsigned long Hysth80m = Thresh80m;
unsigned long Hysth40m = Thresh40m;
unsigned long Hysth30m = Thresh30m;
unsigned long Hysth20m = Thresh20m;
unsigned long Hysth15m = Thresh15m;
byte LPFmode;
byte LastLPFmode = 0;
boolean PA_on = false;

// Keyer
byte cwDelayTime; // in 10 ms steps, how long the tx will hold between cw key downs
int cwSpeed; //this is actuall the dot period in milliseconds
boolean Iambic_Key;
char keyerControl;
char IambicA;

/**
 * The Raduino supports two VFOs : A and B and receiver incremental tuning (RIT). 
 * we define a variables to hold the frequency of the two VFOs, RITs
 * the rit offset as well as status of the RIT
 */
 #define VFO_A 0
 #define VFO_B 1
byte ritOn = 0;            //toggle RIT on/off
byte SplitOn = 0;            //toggle Split on/off
byte CwOn = 0;             //toggle mode CW/SSB
char vfoActive = VFO_A;    //toggle VFO A/B
static unsigned long frequency;
unsigned long VFOfrequency;
int RIToffset=0;
int OrgRIT=0;
boolean txCAT = false;        //turned on if the transmitting due to a CAT command

//we directly generate the CW by programming the Si5351 to the cw tx frequency, hence, both are different modes
//these are the parameter passed to startTx
#define TX_SSB 0 //to differ Si5351 programming mode between SSB and CW
#define TX_CW  1

#define LOWEST_FREQ  (1000000L)
#define HIGHEST_FREQ (30000000L)
byte old_version;

// all variables that will be stored in EEPROM are contained in this 'struct':
struct userparameters {                 // adresse
  byte raduino_version;                 // 4 version identifier
  unsigned long FIRST_IF = 45000000L;   // 5 default value of 1st IF
  unsigned long BFO = 11997450L;        // 9 default calibrated BFO
  unsigned long CW_NARR_FILT = 11997550L; // 13 default calibrated CW filter center frequency
  unsigned long IFshift = 0;      // 17 IF shift
  unsigned long vfoA =7100000L;   // 21
  unsigned long vfoB =14200000L;  // 25
  unsigned int  sideTone = 750;   // 29
  byte EEcwDelayTime = 150;       // 31 in 10 ms steps, how long the tx will hold between cw key downs
  int EEcwSpeed = 100;            // 32 CW the dot period in milliseconds
  boolean EEKeyerIambic = true;   // 34
  char EEIambicAB = 0x00;         // 35
  int VmeterCal = VoltmeterCal;
  byte SmeterCal = SmeterCalDefault;
  //unsigned long freqbird[5]= { 500000L, 500000L,  500000L, 500000L, 500000L };
};
// we can access each of the variables inside the above struct by "User.variablename"
struct userparameters User;


/**
 * Raduino needs to keep track of current state of the transceiver. These are a few variables that do it
 */
byte inTx = 0;
byte isUSB = 127;
int SmeterPrev = 0;
byte TempoSWR = 0;
unsigned int VmeterPrev = 0;
unsigned long cwTimeout = 0;
byte menuOn = 0; // Defines for menu on=1/off=0
boolean  ToggleDone = false ;
boolean  TUNE = false;
boolean Dispreverse  = false ;
boolean  fastUP = false;
boolean  fastDOWN = false;

/** Tuning Mechanism of the Raduino
 *  We use a linear pot that has two ends connected to +5 and the ground. the middle wiper
 *  is connected to ANALOG_TUNNING pin. Depending upon the position of the wiper, the
 *  reading can be anywhere from 0 to 1023.
 *  The tuning control works in steps of 50Hz.
 *  Hence the turning the pot fully from one end to the other will cover 50 x 900 = 45 KHz.
 *  At the two ends, that is, the tuning starts slowly stepping up or down in 10 KHz steps 
 *  To stop the scanning the pot is moved back from the edge. 
 *  To rapidly change from one band to another, you press the function button and then
 *  move the tuning pot. Now, instead of 50 Hz, the tuning is in steps of 50 KHz allowing you
 *  rapidly use it like a 'bandset' control.
 *  To implement this, we fix a 'base frequency' to which we add the offset that the pot 
 *  points to. We also store the previous position to know if we need to wake up and change
 *  the frequency.
 */
unsigned long baseTune;
int  old_knob = 1030; //to differ after a reset
long LPFhyst  = 0;

// *************  SI5315 routines - tks Jerry Gaffke, KE7ER   ***********************

// An minimalist standalone set of Si5351 routines.
// VCOA is fixed at 875mhz, VCOB not used.
// The output msynth dividers are used to generate 3 independent clocks
// with 1hz resolution to any frequency between 4khz and 109mhz.

// Usage:
// Call si5351bx_init() once at startup with no args;
// Call si5351bx_setfreq(clknum, freq) each time one of the
// three output CLK pins is to be updated to a new frequency.
// A freq of 0 serves to shut down that output clock.

// The global variable si5351bx_vcoa starts out equal to the nominal VCOA
// frequency of 25mhz*35 = 875000000 Hz.  To correct for 25mhz crystal error,
// the user can adjust this value.  The vco frequency will not change but
// the number used for the (a+b/c) output msynth calculations is affected.
// Example:  We call for a 5mhz signal, but it measures to be 5.001mhz.
// So the actual vcoa frequency is 875mhz*5.001/5.000 = 875175000 Hz,
// To correct for this error:     si5351bx_vcoa=875175000;

// Most users will never need to generate clocks below 500khz.
// But it is possible to do so by loading a value between 0 and 7 into
// the global variable si5351bx_rdiv, be sure to return it to a value of 0
// before setting some other CLK output pin.  The affected clock will be
// divided down by a power of two defined by  2**si5351_rdiv
// A value of zero gives a divide factor of 1, a value of 7 divides by 128.
// This lightweight method is a reasonable compromise for a seldom used feature.

void si5351bx_init() {                  // Call once at power-up, start PLLA
  uint8_t reg;  uint32_t msxp1;
  Wire.begin();
  i2cWrite(149, 0);                     // SpreadSpectrum off
  i2cWrite(3, si5351bx_clken);          // Disable all CLK output drivers
  i2cWrite(183, SI5351BX_XTALPF << 6);  // Set 25mhz crystal load capacitance
  msxp1 = 128 * SI5351BX_MSA - 512;     // and msxp2=0, msxp3=1, not fractional
  uint8_t  vals[8] = {0, 1, BB2(msxp1), BB1(msxp1), BB0(msxp1), 0, 0, 0};
  i2cWriten(26, vals, 8);               // Write to 8 PLLA msynth regs
  i2cWrite(177, 0x20);                  // Reset PLLA  (0x80 resets PLLB)
  // for (reg=16; reg<=23; reg++) i2cWrite(reg, 0x80);    // Powerdown CLK's
  // i2cWrite(187, 0);                  // No fannout of clkin, xtal, ms0, ms4
}

void si5351bx_setfreq(uint8_t clknum, uint32_t fout) {  // Set a CLK to fout Hz
  uint32_t  msa, msb, msc, msxp1, msxp2, msxp3p2top;
  if ((fout < 500000) || (fout > 109000000)) // If clock freq out of range
    si5351bx_clken |= 1 << clknum;      //  shut down the clock
  else {
    msa = si5351bx_vcoa / fout;     // Integer part of vco/fout
    msb = si5351bx_vcoa % fout;     // Fractional part of vco/fout
    msc = fout;             // Divide by 2 till fits in reg
    while (msc & 0xfff00000) {
      msb = msb >> 1;
      msc = msc >> 1;
    }
    msxp1 = (128 * msa + 128 * msb / msc - 512) | (((uint32_t)si5351bx_rdiv) << 20);
    msxp2 = 128 * msb - 128 * msb / msc * msc; // msxp3 == msc;
    msxp3p2top = (((msc & 0x0F0000) << 4) | msxp2);     // 2 top nibbles
    uint8_t vals[8] = { BB1(msc), BB0(msc), BB2(msxp1), BB1(msxp1),
                        BB0(msxp1), BB2(msxp3p2top), BB1(msxp2), BB0(msxp2)
                      };
    i2cWriten(42 + (clknum * 8), vals, 8); // Write to 8 msynth regs
    i2cWrite(16 + clknum, 0x0C | si5351bx_drive[clknum]); // use local msynth
    si5351bx_clken &= ~(1 << clknum);   // Clear bit to enable clock
  }
  i2cWrite(3, si5351bx_clken);        // Enable/disable clock
}

void i2cWrite(uint8_t reg, uint8_t val) {   // write reg via i2c
  Wire.beginTransmission(SI5351BX_ADDR);
  Wire.write(reg);
  Wire.write(val);
  Wire.endTransmission();
}

void i2cWriten(uint8_t reg, uint8_t *vals, uint8_t vcnt) {  // write array
  Wire.beginTransmission(SI5351BX_ADDR);
  Wire.write(reg);
  while (vcnt--) Wire.write(*vals++);
  Wire.endTransmission();
}
// *********** End of Jerry's si5315bx routines *********************************************************

// création du bargraph dans la mémoire du LCD
char meter[17];
byte s_meter[] = {
  B0,B0,B0,B10101,B0,B0,B0,B0,
  B0,B0,B10000,B10101,B10000,B0,B0,B0,
  B0,B0,B11000,B11101,B11000,B0,B0,B0,
  B0,B0,B11100,B11101,B11100,B0,B0,B0,
  B0,B0,B11110,B11111,B11110,B0,B0,B0,
  B0,B0,B11111,B11111,B11111,B0,B0,B0,
};
void setupSmeter(){ //setup defined characters  with above table in LCD for Smeter bargraph
  lcd.createChar(1, s_meter);
  lcd.createChar(2, s_meter + 8);
  lcd.createChar(3, s_meter + 16);
  lcd.createChar(4, s_meter + 24);
  lcd.createChar(5, s_meter + 32);
  lcd.createChar(6, s_meter + 40);
}


void active_delay(int delay_by){
  unsigned long timeStart = millis();
  while (millis() - timeStart <= delay_by) {
      //Background Work      
    checkCAT();
  }
}


void AutoSetULSB(unsigned long freqsetMode){
      if (freqsetMode > 10000000L){
      if (isUSB != 1)
        si5351bx_setfreq(OutLO2, User.FIRST_IF + User.BFO + User.IFshift );
      isUSB = 1; //USB mode 
    }
    else{
      if (isUSB != 0)
        si5351bx_setfreq(OutLO2, User.FIRST_IF - User.BFO - User.IFshift );
      isUSB = 0; //LSB mode 
    }
}




void setFrequency(unsigned long freqset){
  si5351bx_setfreq(OutLO1, User.FIRST_IF + freqset + RIToffset); //Set frequency LO1 only in Rx

  if (freqset > Hysth160m){ 
     Hysth160m = Thresh160m-5000L;
     LPFmode = 80;
  }
  else {
    Hysth160m = Thresh160m+5000L; // 160m
    LPFmode = 160;
  }
  if (freqset > Hysth80m){ 
     Hysth80m = Thresh80m-5000L;
     LPFmode = 40;
  }
  else {
    Hysth80m = Thresh80m+5000L; // 80m
  }
  if (freqset > Hysth40m){ 
     Hysth40m = Thresh40m-5000L;
     LPFmode = 30;
  }
  else {
    Hysth40m = Thresh40m+5000L; // 40m
  }
  if (freqset > Hysth30m){ 
     Hysth30m = Thresh30m-5000L;
     LPFmode = 20;
  }
  else {
    Hysth30m = Thresh30m+5000L; // 30m
  }
  if (freqset > Hysth20m){ 
     Hysth20m = Thresh20m-5000L;
     LPFmode = 15;
  }
  else {
    Hysth20m = Thresh20m+5000L; // 20m
  }
  if (freqset > Hysth15m){ 
     Hysth15m = Thresh15m-5000L;  //10m

     LPFmode = 10;
  }
  else {
    Hysth15m = Thresh15m+5000L; // 15m
  }
  
if (LastLPFmode != LPFmode){
  LastLPFmode = LPFmode;
  if (LPFmode==80 || LPFmode==160){
     digitalWrite(P3D4, 0); //4 MHz LPF
     digitalWrite(P3D5, 0);
  }
  if (LPFmode==40){
     digitalWrite(P3D4, 1); //7.5 MHz LPF
     digitalWrite(P3D5, 1);
  }
  if (LPFmode==20 || LPFmode==30){
     digitalWrite(P3D4, 1); //15 MHz LPF
     digitalWrite(P3D5, 0 );
  }
  if (LPFmode==10 || LPFmode==15){
     digitalWrite(P3D4, 0); //direct mode 10m
     digitalWrite(P3D5, 1);
  }

}
  Wire.beginTransmission(PaAddress); // transmit to device adress
  Wire.write(LPFmode);       // sends one byte  
  if (!Wire.endTransmission()){
  PA_on = true;
  }
  else {
  PA_on = false;
  }
}


void resetBasefrequency(){
  int knob = analogRead(ANALOG_TUNING);
  baseTune = frequency - (50L * knob);
}



void checkPTT(){  
  //we don't check for ptt when transmitting cw
  if (cwTimeout > 0)
    return;
  if (digitalRead(PTT) == LOW && inTx == 0){
    startTx(TX_SSB);
    active_delay(50); //debounce the PTT
  }
   if (digitalRead(PTT) == HIGH && inTx == 1)
    stopTx();
}


void startTx(byte txMode){
    long DeltaVFO;
    inTx = 1;
    digitalWrite(TX_RX, 1);
    digitalWrite(FBUTTON, LOW); // deactivate pullup
    active_delay(2); //wait to mute réduit de 5 à 2
    
    VFOfrequency = frequency;//vfo actif pour mode split
      if (SplitOn){
          if (vfoActive == VFO_B){
            DeltaVFO = User.vfoA - frequency; //bloque le décalage de fréquence si > 100 kHz
            DeltaVFO = abs(DeltaVFO);
            if (DeltaVFO < 100001L){
              frequency = User.vfoA;
            }
          }
          else {
            DeltaVFO = User.vfoB - frequency;
            DeltaVFO = abs(DeltaVFO);
            if (DeltaVFO < 100001L){
              frequency = User.vfoB;
            }
          }
      }
      if (txMode == TX_SSB){
      si5351bx_setfreq(OutLO1, User.FIRST_IF + frequency); ///in Tx whithout RIT offset
      //digitalWrite(CW_FILTER, 0);//SSB filter anyway à supprimer !
        if (isUSB == 1)   si5351bx_setfreq(OutLO2, User.FIRST_IF + User.BFO );
        else      si5351bx_setfreq(OutLO2, User.FIRST_IF - User.BFO );
      si5351bx_setfreq(OutBFO, User.BFO ); // BFO only
    }
    if (txMode == TX_CW){
      si5351bx_setfreq(OutLO1, User.FIRST_IF + frequency);
      si5351bx_setfreq(OutLO2, User.FIRST_IF + User.CW_NARR_FILT); 
      si5351bx_setfreq(OutBFO,User.CW_NARR_FILT); 
    }
    updateDisplay();
}


void stopTx(){
    digitalWrite(TX_RX, 0); //swtich off transmitting
    inTx = 0; //we are now in Rx mode
      pinMode(FBUTTON, INPUT);
      digitalWrite(FBUTTON, HIGH); // activate pullup
    frequency = VFOfrequency;//vfo actif pour mode split
    if (CwOn == 0){ // Rx SSB mode
    si5351bx_setfreq(OutLO1, User.FIRST_IF + frequency + RIToffset); //set LO1
    si5351bx_setfreq(OutBFO, User.BFO + User.IFshift);                //set BFO
      if (isUSB == 1)
        si5351bx_setfreq(OutLO2, User.FIRST_IF + User.BFO + User.IFshift );// set LO2 
      else
        si5351bx_setfreq(OutLO2, User.FIRST_IF - User.BFO - User.IFshift );
    }
    else { // Rx CW mode
          //switch on the second oscillator and the bfo
      si5351bx_setfreq(OutLO1, User.FIRST_IF + frequency + RIToffset);
      si5351bx_setfreq(OutLO2, User.FIRST_IF + User.CW_NARR_FILT );
      si5351bx_setfreq(OutBFO,User.CW_NARR_FILT + User.sideTone );
    }
    updateDisplay();
}



byte btnDown(){
  if (digitalRead(FBUTTON) == HIGH)
    return 0;
  else
    return 1;
}


void checkButton(){
  int i, t1, knob, new_knob, duration;
  long croasefreq;
  //only if the button is pressed
  if (!btnDown())
    return;
  //wait for 50 ms before declaring the button to be really down 
  active_delay(50);
  if (!btnDown())
    return;
    
  knob = analogRead(ANALOG_TUNING);
  duration = 0;
  /* keep measuring how long the duration of btn down has been to a max of 3 seconds */
  while (btnDown() && duration < 3000){
        /* if the tuning knob is moved while the btn is down, 
           then track the bandset until the button is up and return
           LOCKED IF RIT   */
      new_knob = analogRead(ANALOG_TUNING);
      if (abs(new_knob - knob) > 10){
        /* track the tuning and return */
        while (btnDown()){
          if (ritOn == 0){
            croasefreq = ((analogRead(ANALOG_TUNING) * 30000L) + 1000000L);    
            if (croasefreq <= HIGHEST_FREQ) {
            frequency = baseTune = croasefreq;
            }
          }
         else{
           lcd.setCursor(0, 1);
           lcd.print(F("RIT:On -LOCKED-"));
         }
          AutoSetULSB(frequency);
          setFrequency(frequency);
          updateDisplay();
          active_delay(10);
        }
        active_delay(1000);
        return;
      } /* end of handling the bandset */
      
     active_delay(100);
     duration += 100;
  }
  if  (duration == 3000){
      if (vfoActive == VFO_A){
        User.vfoA = frequency;
        User.vfoB = User.vfoA;
      }
      else {
        User.vfoB = frequency;
        User.vfoA = User.vfoB;
      }
      lcd.setCursor(0, 1);
      lcd.print(F("VFO A=B         "));
  }
   
  while (btnDown() && duration < 5000){  // wait until btn is released
     active_delay(100);
     duration += 100;
  }
  if  (duration == 5000){
      lcd.setCursor(0, 1);
      lcd.print(F("VFO A=B EE stored"));
      EEPROM.put(4, User); // save all user parameters to EEPROM
      active_delay(1000);
      while (btnDown()){  // wait until btn is released
        active_delay(10);
      }
      return;
  }
  
  if  (duration < 1000) {
    doMenu();
  }
}


void doTuning(){
     byte i;
     int s, knob = 0;
  //do a peak detection in Rx mode
  for (i = 0; i < 127; i++){
    s = analogRead(ANALOG_TUNING); //sample tunning pot
    if (s > knob)                  //peak detection
      knob = s;
    if (!digitalRead(PTT))
    break;
  }

  if (knob < 70) fastDOWN = true;
  else fastDOWN = false;
  if (knob > 950) fastUP = true;
  else fastUP = false;
  
 if (knob < 10 && frequency > LOWEST_FREQ) { //réglage rapide bas
  if (!ritOn){
      baseTune = baseTune - 1000L;
      frequency = baseTune;
  }
      AutoSetULSB(frequency);
      setFrequency(frequency);
      updateDisplay();
      active_delay(30);
  } 
  else if (knob > 1010 && frequency < HIGHEST_FREQ) { // réglage rapide haut
    if (!ritOn){
     baseTune = baseTune + 1000L; 
     frequency = baseTune + 50500l;
    }
      AutoSetULSB(frequency);
     setFrequency(frequency);
     updateDisplay();
     active_delay(30);
  }
  // in the middle, it is normal tuning
  else if (knob != old_knob){
    if (!ritOn){
     frequency = baseTune + (50L * knob);
    }
    else{
     RIToffset = (knob  - OrgRIT)*50;
     if (RIToffset > 10000) RIToffset = 10000;
     if (RIToffset < -10000) RIToffset = -10000;
    }
     old_knob = knob;
     setFrequency(frequency);
     updateDisplay();
  }
}



//********* SETUP at power reset   ********
void setup()
{
  User.raduino_version = VersID;  // byte max 255
  lcd.begin(16, 2);
  lcd.print(F("VFOduino "));
  lcd.print(F(VERSION));
  lcd.setCursor(0, 1);
  lcd.print(F("Rocket F5RCT"));
  setupSmeter();
  
  // Start serial
  Serial.begin(38400); ///à passer en 38400 pour le mode CAT
  Serial.flush();  // vide le buffer 
  Serial.println("Rocket OK");

  analogReference(DEFAULT);
  pinMode(FBUTTON, INPUT);
  digitalWrite(FBUTTON, HIGH); // activate pullup
  pinMode(PTT, INPUT);
  digitalWrite(PTT, HIGH);  // activate pullup
  pinMode(CW_TONE, OUTPUT);  
  digitalWrite(CW_TONE, 0);
  pinMode(TX_RX,OUTPUT);
  digitalWrite(TX_RX, 0);
  pinMode(P3D5, OUTPUT);
  digitalWrite(P3D5, 1);
  pinMode(P3D4, OUTPUT);
  digitalWrite(P3D4, 1);
  pinMode(CW_FILTER, OUTPUT);
  digitalWrite(CW_FILTER, 0);
  pinMode(CW_CARRIER, OUTPUT);
  digitalWrite(CW_CARRIER, 0);

  si5351bx_init();
  active_delay(3000);
  // when Fbutton is kept pressed during power up, or after a version update
  // then all settings will be restored to the standard "factory" values

  EEPROM.get(4, old_version); // previous sketch version
  if (digitalRead(FBUTTON)== LOW || (old_version != User.raduino_version)) {
    factory_settings();
    while(btnDown());   // attente bouton relaché
  }

  //retrieve user settings from EEPROM
  EEPROM.get(4, User);
  frequency = User.vfoA;
  resetBasefrequency();
  cwDelayTime = User.EEcwDelayTime;
  Iambic_Key = User.EEKeyerIambic;
  cwSpeed = User.EEcwSpeed;
  keyerControl = User.EEIambicAB;
  long SiCAL = 0;                     // Si5351 calibration +/- 0.3 ppm offset on base frequency
  EEPROM.get(0, SiCAL);
  si5351bx_vcoa = (SI5351BX_XTAL * SI5351BX_MSA) + SiCAL; // apply the calibration correction factor
  si5351bx_setfreq(OutBFO, User.BFO + User.IFshift); //default mode is SSB
  AutoSetULSB(frequency);
  Serial.println("Setup Done");
}

//**************** MAIN LOOP *************************
void loop(){
  if (CwOn == 1)  //the order of testing first for cw and then for ptt is important.
     cwKeyer(); 
  else{
    active_delay(34); //délai dans la boucle principale en Rx/Tx SSB uniquement
      if (!txCAT)  checkPTT(); // enable PTT only in SSB mode and not in CAT
  }

  if (!inTx){
    checkButton(); // check button or tune only when not tranmsitting
    doTuning();
    
  }
  updateMeter();
  checkCAT();   //we check CAT after tuning as it might put the radio into TX
}

// ****************** WASTE ZONE - UNUSED functions *******************************
/*




*/
