/**
 * 
 CW Key logic change with ron's code (ubitx_keyer.cpp)
 Ron's logic has been modified to work with the original uBITX by KD8CEC
 Then modified without any "while" by F5RCT

 Original Comment ----------------------------------------------------------------------------
 * The CW keyer handles either a straight key or an iambic / paddle key.
 * They all use just one analog input line. This is how it works.
 * The analog line has the internal pull-up resistor enabled. 
 * When a straight key is connected, it shorts the pull-up resistor, analog input is 0 volts
 * When a paddle is connected, the dot and the dash are connected to the analog pin through
 * a 10K and a 2.2K resistors. These produce a 4v and a 2v input to the analog pins.
 * So, the readings are as follows :
 * 0v - straight key
 * paddle dot 2.50V nom
 * paddle dash 3.40V nom
 * dot and dash pressed 2.04V nom
 * The keyer is written to transparently handle all these cases
 * 
 * Generating CW
 * The CW is cleanly generated by unbalancing the BFO mixer
 * and putting the local oscillator directly at the CW transmit frequency.
 * The sidetone, generated by the Arduino is injected into the volume control
 */

 //CW ADC Range
const int cwAdcBothLo = 207; // 1.01V
const int cwAdcBothHi = 463;  // 2.26V
const int cwAdcDotHi = 604;   //  2.95V
const int cwAdcDashHi = 859;  //4.2V

byte delayBeforeCWStartTime = 10; //retard de passage en mode TxCW avant d'envoyer la porteuse
char lastPaddle = 0;  //we store the last padde's character to alternatively send dots and dashes when both are simultaneously pressed

/* Starts transmitting the carrier with the sidetone, 
 * each time it is called, the cwTimeOut is pushed further into the future  */
void cwKeydown(){
  tone(CW_TONE, User.sideTone); 
  digitalWrite(CW_CARRIER, 1);     
  cwTimeout = millis() + cwDelayTime * 10;  //Modified by KD8CEC, for CW Delay Time save to eeprom
}

/* Stops the cw carrier transmission along with the sidetone  */
void cwKeyUp(){
  noTone(CW_TONE);
  digitalWrite(CW_CARRIER, 0);    
}

//Variables for Ron's new logic
#define DIT_L 0x01 // DIT latch
#define DAH_L 0x02 // DAH latch
#define DIT_PROC 0x04 // DIT is being processed
#define PDLSWAP 0x08 // 0 for normal, 1 for swap
#define IAMBICB 0x10 // 00 for Iambic A, 10 for Iambic B
enum KSTYPE {IDLE, CHK_DIT, CHK_DAH, KEYED_PREP, KEYED, INTER_ELEMENT };
static unsigned long ktimer;
char keyerState = IDLE;

// Read analog input
char update_PaddleLatch(byte isUpdateKeyState) {
char tmpKeyerControl = 0;
int paddle = analogRead(ANALOG_KEYER);
  //itoa(paddle, b, 10);    //diagnostic, VU2ESE
  //printLine1(b);
 if(Iambic_Key){
    if (paddle >= cwAdcDotHi && paddle <= cwAdcDashHi){
      tmpKeyerControl |= DAH_L;
    }
    else if (paddle >= cwAdcBothHi && paddle <= cwAdcDotHi){
     tmpKeyerControl |= DIT_L;
    } 
    else if (paddle >= cwAdcBothLo && paddle <= cwAdcBothHi){  //both
      tmpKeyerControl |= (DAH_L | DIT_L) ;
    }
    else { tmpKeyerControl = 0 ; 
    }
 }
 else {
    if (paddle <= cwAdcDotHi){
        tmpKeyerControl = DIT_L ;
    }
    else { tmpKeyerControl = 0 ; 
    }
 }
  if (isUpdateKeyState == 1)  keyerControl |= tmpKeyerControl;
  return tmpKeyerControl;
}

/*****************************************************************************
// New logic, by RON modified by KD8CEC
******************************************************************************/
void cwKeyer(void){
  lastPaddle = 0;
  char tmpKeyControl = 0;
  if( Iambic_Key ) {
      switch (keyerState) {
        case IDLE:
          tmpKeyControl = update_PaddleLatch(0);
          if ( tmpKeyControl == DAH_L || tmpKeyControl == DIT_L || 
            tmpKeyControl == (DAH_L | DIT_L) || (keyerControl & 0x03)) {
             update_PaddleLatch(1);
             keyerState = CHK_DIT;
          }else{
            if (0 < cwTimeout && cwTimeout < millis()){
              cwTimeout = 0;
              stopTx();
            }
          }
          break;
    
        case CHK_DIT:
          if (keyerControl & DIT_L) {
            keyerControl |= DIT_PROC;
            ktimer = cwSpeed;
            keyerState = KEYED_PREP;
          }else{
            keyerState = CHK_DAH;
          }
          break;
    
        case CHK_DAH:
          if (keyerControl & DAH_L) {
            ktimer = cwSpeed*4; //passé de 3 à 4 23/12/2020
            keyerState = KEYED_PREP;
          }else{
            keyerState = IDLE;
          }
          break;
    
        case KEYED_PREP:
          if (!inTx){
            startTx(TX_CW);
            cwTimeout = millis() + cwDelayTime * 10;  //+ CW_TIMEOUT;
            active_delay(delayBeforeCWStartTime); // délai de  passage Rx->Tx  avant l'envoi d'un point ou d'un trait
          }
          ktimer += millis(); // set ktimer to interval end time
          keyerControl &= ~(DIT_L + DAH_L); // clear both paddle latch bits
          keyerState = KEYED; // next state 
          cwKeydown();
          break;
    
        case KEYED:
          if (millis() > ktimer) { // are we at end of key down ?
           cwKeyUp();
           ktimer = millis() + cwSpeed; // inter-element time
            keyerState = INTER_ELEMENT; // next state
          }else if (keyerControl & IambicA) {
            update_PaddleLatch(1); // early paddle latch in Iambic B mode
          }
          break;
    
        case INTER_ELEMENT:
          // Insert time between dits/dahs
          update_PaddleLatch(1); // latch paddle state
          if (millis() > ktimer) { // are we at end of inter-space ?
            if (keyerControl & DIT_PROC) { // was it a dit or dah ?
              keyerControl &= ~(DIT_L + DIT_PROC); // clear two bits
              keyerState = CHK_DAH; // dit done, check for dah
            }else{
              keyerControl &= ~(DAH_L); // clear dah latch
              keyerState = IDLE; // go idle
            }
          }
          break;
      }
  }
  else{       //straight mode
      if (update_PaddleLatch(0) == DIT_L) { // if the key is pressed
        if (!inTx){   //start Tx
           startTx(TX_CW);
           active_delay(delayBeforeCWStartTime); // délai de passage Rx->Tx avant l'envoi d'un point ou d'un trait
        }
        cwKeydown();
      }
      else{ // the key is released
        cwKeyUp(); 
        if (0 < cwTimeout && cwTimeout < millis()){
          cwTimeout = 0;
          stopTx(); //Tx stop, control by Main Loop
        }
      }
  }   //end of elese iambic/straight
}
